/*
cd /Users/gjoffre/Code/ExrTool/pm_exr/build/Debug/ 
./pm_exr -i "/Users/gjoffre/Documents/valerie.exr" -o "/Users/gjoffre/Documents/dumpvalerie.hdrrgbe" 

*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "ImfCRgbaFile.h"

const int debug=1;


int pmexr_dim(const char *nomfic, int *info) {
	int xMin=0,yMin=0,xMax=0,yMax=0;
	if (debug) fprintf (stderr, "start pmexr_dim\n");

	ImfInputFile *exrIn=ImfOpenInputFile (nomfic);
	//if (debug) printf("apres OpenIn \n");
	const ImfHeader *tete= ImfInputHeader(exrIn);
	ImfHeaderDataWindow (tete, &xMin, &yMin, &xMax, &yMax);	
	info[0]=xMax-xMin+1; info[1]=yMax-yMin+1;
	ImfCloseInputFile (exrIn);
	return 32;
}


int  pmexr_read(const char *nomfic, float *pixDim, unsigned char *rgbe) {
	int j,i,iexp,cnt=0,xMin=0,yMin=0,xMax=0,yMax=0,retour=0;
	float Er,Eg,Eb,Efrac,Emax,Ei,imMin=99999999.0,imMax=0; double imMoy=0;
	ImfInputFile *exrIn=ImfOpenInputFile (nomfic);
	//if (debug) printf("apres OpenIn \n");
	const ImfHeader *tete= ImfInputHeader(exrIn);
	ImfHeaderDataWindow (tete, &xMin, &yMin, &xMax, &yMax);	
	const int iMax=xMax-xMin+1, jMax=yMax-yMin+1, t2=iMax*2,t3=iMax*3; //,datalen=jMax*iMax*3;

	ImfRgba *rgbaBuf=(ImfRgba*)calloc(jMax*iMax,sizeof(ImfRgba));

	retour=ImfInputSetFrameBuffer (exrIn, rgbaBuf, 1,iMax);
	if (debug) printf("retour SetFrameBuffer %d\n",retour);
	retour=ImfInputReadPixels (exrIn, yMin, yMax);
	if (debug) printf("retour Read %d\n",retour);
	ImfCloseInputFile (exrIn);

	ImfRgba *prgba=rgbaBuf; 	unsigned char *pPixels=rgbe;
	for (j=0;j<jMax;j++)	{
		for (i=0;i<iMax;i++) {
			Er=ImfHalfToFloat(prgba[0].r); Eg=ImfHalfToFloat(prgba[0].g); Eb=ImfHalfToFloat(prgba[0].b);
			Ei=0.3*Er+0.6*Eg+0.1*Eb;
			if (Ei>imMax) imMax=Ei; if (Ei>0) if (Ei<imMin) imMin=Ei; imMoy+=Ei; cnt++;
			
			Emax=Er; if (Eg>Emax) Emax=Eg; if (Eb>Emax) Emax=Eb;
			if (Emax>1e-32) {
				Efrac=frexp(Emax,&iexp) * 256.0/Emax;
				pPixels[0]=(unsigned char) (Er*Efrac);
				pPixels[iMax]=(unsigned char) (Eg*Efrac);
				pPixels[t2]=(unsigned char) (Eb*Efrac);
				pPixels[t3]=(unsigned char) (iexp+128);
			}
			else pPixels[0]=pPixels[iMax]=pPixels[t2]=pPixels[t3]=0;
			pPixels++; prgba++;
		}
		pPixels+=t3;
	}
	free(rgbaBuf);
	pixDim[0]=256.0*imMin; pixDim[1]=256.0*imMax; 
	if (cnt>0) pixDim[2]=256.0*imMoy/(float)cnt; 
	return retour;
}


int main (int argc, const char * argv[]) {
	int arg=0, valid=0, retour=0;
	char opt;	const char *fullOption;	
	const char *inFileName=NULL,  *outFileName=NULL; 
	FILE *ofp;
	if (debug) fprintf (stderr, "argc: %d\n",argc);
	
	//for (arg=1; argv[arg][0] == '-';) {
	for (arg=1; arg<argc;) {
		//if (debug) fprintf (stderr, "arg: %d\n",arg);
		valid=1;
		fullOption=argv[arg];
		if (argv[arg][0] == '-') {
			opt=argv[arg++][1];  		if (debug) fprintf (stderr, "opt: %c\n",opt);
			switch (opt)    {
				case 'i':  inFileName = argv[arg++];  break;
				
				case 'o':  outFileName = argv[arg++];  break;

				default:  valid=0;
			}
			if (!valid)		{
				fprintf(stderr, "Unknown option \"%s\".\n", fullOption);
				return 1;
			}
		}
	}
	
	if (inFileName==NULL)	{
		fprintf(stderr, "No input file.\n");
		return 1;
	}	
	if (outFileName==NULL)	{
		fprintf(stderr, "No output file.\n");
		return 1;
	}	
	
	int *exrInfo=calloc(2, sizeof(int));
	//if (debug) fprintf (stderr, "before call pmexr_dim\n");
	retour=pmexr_dim(inFileName, exrInfo);
	int wImage=exrInfo[0], hImage=exrInfo[1];  free(exrInfo);  	if (debug) fprintf (stderr, "%d x %d\n",wImage, hImage);

	if ((wImage<10)||(hImage<10)) return -2;

	unsigned char *rgbe=calloc(wImage*hImage, 4);  if (rgbe==NULL) return -4;
	float *pixDim=calloc(3, sizeof(float)); 
	
	retour=pmexr_read(inFileName, pixDim, rgbe);			
    
	if (debug) fprintf(stderr, "pixDim:  %1.2f  %1.2f  %1.2f\n",pixDim[0], pixDim[1], pixDim[2]);
	free(pixDim);

    if ((ofp = fopen(outFileName, "wb")) == NULL) {   fprintf(stderr, "can't open %s\n", outFileName);    return -1;  }

	retour=fwrite(rgbe, wImage*hImage, 4, ofp);
	free(rgbe);
	if (fwrite(rgbe, wImage*hImage, 4, ofp)!=4) {  fprintf(stderr, "fwrite rgbe failed - returned: %d\n",retour); return -3; }
	fclose(ofp);
	
	return 0;
}

